<?php
/*
Plugin Name: ComPreVal
Plugin URI: http://dev.d10e.net/files/compreval/
Description: Adds serverside comment preview and validation to WordPress
Version: 1.0
Author: Mark Wubben & Ben de Groot
Author URI: http://dev.d10e.net/
*/ 

/*
	Most of the work on this has been done by Mark,
	Ben just did some minor changes and published this with Mark's permission
	Published under the GNU General Public License:
	http://www.opensource.org/licenses/gpl-license.php
*/

// requires SafeHtml
$comment_preview_validate = true;

// validating comment if posted (to prevent bypassing of validation)
if($comment_preview_validate &&  preg_match("/wp-comments-post\.php/", $_SERVER['REQUEST_URI'])){
	if(!class_exists("SafeHtmlChecker")){
		include(ABSPATH . 'wp-content/plugins/safehtml.php');
	}

	$comment = trim($HTTP_POST_VARS['comment']);
	$author = trim(strip_tags($_POST['author']));
	$email = trim(strip_tags($_POST['email']));

	if($comment == "" || preg_match("/(<p>\s*\n*<\/p>|<br \/>)/", $comment)){
		die( __('Error: please type a comment.') );
	}

	if(get_settings('require_name_email')){
		if(preg_match("/^[a-zA-Z][\w\.-]*[a-zA-Z0-9]@[a-zA-Z0-9][\w\.-]*[a-zA-Z0-9]\.[a-zA-Z][a-zA-Z\.]*[a-zA-Z]$/", $email) == false || $author == ""){
			die("Please fill in the name, email and comment fields.");
		}
	}

	$comment = wpautop($comment, 0);
	$comment = balanceTags($comment, 1);
	$comment = format_to_post($comment);
	$comment = apply_filters('post_comment_text', $comment);
	$checker = new SafeHtmlChecker;
	$comment = stripslashes($comment);
	$checker->check('<all>'.$comment.'</all>');
	if(!$checker->isOK()) {
		die("You have tried to bypass the preview and submit an invalid comment. Needless to say, you have failed.");
	}
}

// remove WP validator
remove_filter('comment_text', 'wp_filter_kses');

// actual code
function comment_preview(){
	if(isset($_POST["preview_submit"])){
		comment_preview_show();
	} else {
		comment_preview_new();
	}
}

function comment_preview_show(){

	global $require_name_email, $comment_preview_validate;
	
	if($comment_preview_validate && !class_exists("SafeHtmlChecker")){
		die("Unable to validate comment, SafeHtmlChecker is not loaded.");
	} else {
		$bValidate = true;
	}
	
	$author = trim(htmlspecialchars($_POST["preview_author"]));
	$email = trim(htmlspecialchars($_POST["preview_email"]));
	$url = trim(htmlspecialchars($_POST["preview_url"])); 
		/* check url for http, in case base url is in use */
		$urlstart = substr($url,0,7);
		if ("http://" !== $urlstart) {
			$url = "http://$url";
			} 
	$comment = trim($_POST["preview_comment"]);
	$id = intval($_POST['preview_comment_post_ID']);

	$redirect = trim($_POST["preview_redirect_to"]);

	$comment = wpautop($comment, 0);
	$comment = balanceTags($comment, 1);
	$comment = format_to_post($comment);
	$comment = apply_filters('post_comment_text', $comment);
	$comment = stripslashes($comment);
	$comment = trim($comment);
	if(preg_match("/(<p>\s*\n*<\/p>|<br \/>)/", $comment)){
		$comment = "";
	}

	if($bValidate){
		$checker = new SafeHtmlChecker;
		$checker->check('<all>'.$comment.'</all>');
		if($checker->isOK()) {
			$bIsValid = true;
		} else {
			$bIsValid = false;	
		}
	} else {
		$bIsValid = true;
	}

	if(get_settings('require_name_email')){
		if(preg_match("/^[a-zA-Z][\w\.-]*[a-zA-Z0-9]@[a-zA-Z0-9][\w\.-]*[a-zA-Z0-9]\.[a-zA-Z][a-zA-Z\.]*[a-zA-Z]$/", $email) != false && $author != ""){
			$details = true;
		} else {
			$details = false;
		}
	} else {
		$details = true;
	}

	/* Outputted HTML */
	echo "<h3 id=\"preview\">Your comment preview</h3>\n";	
	if($bIsValid && $details && $comment != "comment" && $comment != ""){
		if($url != ""){
			echo "<p><a href=\"$url\">$author</a>";
		} else {
			echo $author;
		}
		echo " says:</p>\n<div id=\"previewedcomment\">\n";
		echo $comment;
		echo "\n</div>\n<form action=\"".get_settings("siteurl")."/wp-comments-post.php\" method=\"post\">\n<div>";
		echo "<input type=\"hidden\" name=\"comment_post_ID\" value=\"$id\" />";
		echo "<input type=\"hidden\" name=\"redirect_to\" value=\"$redirect\" />";	
		echo "<input type=\"hidden\" name=\"author\" value=\"$author\" />";
		echo "<input type=\"hidden\" name=\"email\" value=\"$email\" />";
		echo "<input type=\"hidden\" name=\"url\" value=\"$url\" />";
		echo "<input type=\"hidden\" name=\"comment\" value=\"".htmlspecialchars($comment)."\" />";
		echo "<p><input type=\"submit\" name=\"submit\" id=\"submitComment\" value=\"Submit\"/></p>";
		echo "</div>\n</form>\n";
	} elseif(!$bIsValid) {
		echo "\n<p>Unfortunately your comment is not well-formed according to the XML rules or contains illegal tags and/or attributes. Here is a list of the errors found while validating your comment:</p>\n";
		echo '<ul>'."\n";
		foreach ($checker->getErrors() as $error) {
			echo '<li>'.$error.'</li>'."\n";
		}
		echo '</ul>'."\n";
	} else {
		echo "\n<p>Please fill in the name, email and comment fields.</p>\n";
	}
	
	echo "<h3 id=\"editcomment\">Edit your comment</h3>\n";
	echo "<form action=\"".$_SERVER['REQUEST_URI']."#preview\" method=\"post\">\n";
	echo "<div>\n";
	echo "<p><label for=\"comment\">Your Comment</label>\n";
	echo "<textarea name=\"preview_comment\" id=\"comment\" cols=\"50\" rows=\"10\">".htmlspecialchars($comment)."</textarea></p>\n";
 	echo "<p><input type=\"text\" name=\"preview_author\" id=\"author\" value=\"$author\" />\n";
 	echo "<label for=\"author\">Name</label>\n";
	echo "<input type=\"hidden\" name=\"preview_comment_post_ID\" value=\"$id\" />\n";
	echo "<input type=\"hidden\" name=\"preview_redirect_to\" value=\"$redirect\" /></p>\n";
	echo "<p><input type=\"text\" name=\"preview_email\" id=\"email\" value=\"$email\" />\n";
	echo "<label for=\"email\">Email</label></p>\n";
	echo "<p><input type=\"text\" name=\"preview_url\" id=\"url\" value=\"$url\" />\n";
	echo "<label for=\"url\">Website</label></p>\n";
	echo "<p><input name=\"preview_submit\" type=\"submit\" id=\"submitComment\" value=\"Preview\" /></p>\n";
	echo "</div>\n";
	echo "</form>\n";

}

function comment_preview_new(){

	global $post, $comment_author, $comment_author_email, $comment_author_url;
	
	echo "<div id=\"postcomment\">\n<h3 id=\"respond\">Leave a Reply</h3>\n";
	echo "<p>Common XHTML tags allowed, which will be validated. Spam and trolling will not be tolerated.</p>\n";
	echo "<form action=\"".$_SERVER['REQUEST_URI']."#preview\" method=\"post\">\n";
	echo "<div>\n";
	echo "<p><label for=\"comment\">Your Comment</label>\n";
	echo "<textarea name=\"preview_comment\" id=\"comment\" cols=\"50\" rows=\"10\"></textarea></p>\n";
	echo "<p><input type=\"text\" name=\"preview_author\" id=\"author\" value=\"$comment_author\" />\n";
	echo "<label for=\"author\">Name</label>\n";
	echo "<input type=\"hidden\" name=\"preview_comment_post_ID\" value=\"".$post->ID."\" />\n";
	echo "<input type=\"hidden\" name=\"preview_redirect_to\" value=\"".htmlspecialchars($_SERVER["REQUEST_URI"])."#comments\" /></p>\n";
	echo "<p><input type=\"text\" name=\"preview_email\" id=\"email\" value=\"$comment_author_email\" />\n";
	echo "<label for=\"email\">Email (will not be published)</label></p>\n";
	echo "<p><input type=\"text\" name=\"preview_url\" id=\"url\" value=\"$comment_author_url\" />\n";
	echo "<label for=\"url\">Website</label></p>\n";
	echo "<p><input name=\"preview_submit\" type=\"submit\" id=\"submitComment\" value=\"Preview\" /></p>\n";
	echo "</div>\n";
	echo "</form>\n</div>";
 
}
?>
